﻿using Microsoft.VisualStudio.Shell;
using Microsoft.VisualStudio.Shell.Interop;
using System;
using System.ComponentModel.Design;
using System.Globalization;

namespace TestProject
{
    /// <summary>
    /// 테스트 명령
    /// </summary>
    internal sealed class TestCommand
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 명령 ID
        /// </summary>
        public const int CommandID = 0x0100;

        /// <summary>
        /// 명령 세트
        /// </summary>
        public static readonly Guid CommandSet = new Guid("EE73DD14-8767-4596-9AB9-7C271A43C6F5");

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 패키지
        /// </summary>
        private readonly AsyncPackage package;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 인스턴스 - Instance

        /// <summary>
        /// 인스턴스
        /// </summary>
        public static TestCommand Instance
        {
            get;
            private set;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 서비스 공급자 - ServiceProvider

        /// <summary>
        /// 서비스 공급자
        /// </summary>
        private Microsoft.VisualStudio.Shell.IAsyncServiceProvider ServiceProvider
        {
            get
            {
                return this.package;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 생성자 - TestCommand(package, commandService)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="package">패키지</param>
        /// <param name="commandService">명령 서비스</param>
        private TestCommand(AsyncPackage package, OleMenuCommandService commandService)
        {
            this.package = package ?? throw new ArgumentNullException(nameof(package));

            commandService = commandService ?? throw new ArgumentNullException(nameof(commandService));

            CommandID commandID = new CommandID(CommandSet, CommandID);

            MenuCommand menuCommand = new MenuCommand(this.Execute, commandID);

            commandService.AddCommand(menuCommand);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 초기화하기 (비동기) - InitializeAsync(package)

        /// <summary>
        /// 초기화하기 (비동기)
        /// </summary>
        /// <param name="package">패키지</param>
        /// <returns>태스크</returns>
        public static async System.Threading.Tasks.Task InitializeAsync(AsyncPackage package)
        {
            await ThreadHelper.JoinableTaskFactory.SwitchToMainThreadAsync(package.DisposalToken);

            OleMenuCommandService commandService = await package.GetServiceAsync(typeof(IMenuCommandService)) as OleMenuCommandService;

            Instance = new TestCommand(package, commandService);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 실행하기 - Execute(sender, e)

        /// <summary>
        /// 실행하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Execute(object sender, EventArgs e)
        {
            ThreadHelper.ThrowIfNotOnUIThread();

            string message = string.Format(CultureInfo.CurrentCulture, "Inside {0}.MenuItemCallback()", this.GetType().FullName);
            string title   = "TestCommand";

            VsShellUtilities.ShowMessageBox
            (
                this.package,
                message,
                title,
                OLEMSGICON.OLEMSGICON_INFO,
                OLEMSGBUTTON.OLEMSGBUTTON_OK,
                OLEMSGDEFBUTTON.OLEMSGDEFBUTTON_FIRST
            );
        }

        #endregion
    }
}